import { IPCClient } from 'volans-common';
import { EventEmitter } from 'volans-common';
let requestId = 1;
function countRequestId() {
    return requestId++;
}
export class Client {
    constructor(pipeFile) {
        this._callbacks = new Map();
        this._onMessageEvent = new EventEmitter();
        this.onMessageEvent = this._onMessageEvent.event;
        this._pipeFile = pipeFile;
        this._ipcClient = new IPCClient();
        this._protocol = this._ipcClient.connectTo(pipeFile);
        this._onReturnEvent();
    }
    get pipeFile() {
        return this._pipeFile;
    }
    send(params) {
        const id = countRequestId();
        this._protocol.send(Object.assign({}, params, { requestId: id }));
        return id;
    }
    runProtocol(method, payload, token) {
        return new Promise(resolve => {
            const id = this.send({ method, payload });
            this._callbacks.set(id, (payload) => {
                resolve(payload);
            });
            if (token) {
                token.cancel = () => {
                    this._callbacks.delete(id);
                    resolve(null);
                };
            }
        });
    }
    _onReturnEvent() {
        this._protocol.onMessage((event) => {
            const { requestId, returnValue, method, payload } = event;
            /**
             * 拥有 requestId 一般为返回消息。
             */
            if (!method) {
                const callback = this._callbacks.get(requestId);
                this._callbacks.delete(requestId);
                if (callback) {
                    callback(returnValue);
                }
            }
            else {
                this._onMessageEvent.fire({ method, payload });
            }
        });
    }
}
